@@ -192,6 +192,9 @@ class Application(object):
         except cliapp.UnknownConfigVariable, e:  # pragma: no cover
             stderr.write('ERROR: %s\n' % str(e))
             sys.exit(1)
+        except cliapp.MalformedYamlConfig as e:  # pragma: no cover
+            stderr.write('ERROR: %s\n' % str(e))
+            sys.exit(1)
         except AppException, e:
             log(traceback.format_exc())
             stderr.write('ERROR: %s\n' % str(e))
@@ -402,6 +405,8 @@ class Application(object):
 
         if self.settings['log'] == 'syslog':
             handler = self.setup_logging_handler_for_syslog()
+        elif self.settings['log'] == 'stderr':
+            handler = self.setup_logging_handler_for_stderr()
         elif self.settings['log'] and self.settings['log'] != 'none':
             handler = self.setup_logging_handler_for_file()
         else:
@@ -428,6 +433,20 @@ class Application(object):
         fmt = progname + ": %(levelname)s %(message)s"
         return logging.Formatter(fmt)
 
+    def setup_logging_handler_for_stderr(self):  # pragma: no cover
+        '''Setup a logging.Handler for logging to stderr.'''
+
+        handler = logging.StreamHandler()
+        formatter = self.setup_logging_formatter_for_stderr()
+        handler.setFormatter(formatter)
+
+        return handler
+
+    def setup_logging_formatter_for_stderr(self):  # pragma: no cover
+        '''Setup a logging.Formatter for stderr.'''
+        fmt = "%(levelname)s %(message)s"
+        return logging.Formatter(fmt)
+
     def setup_logging_handler_for_file(self):  # pragma: no cover
         '''Setup a logging handler for logging to a named file.'''
 
