@@ -21,6 +21,15 @@ import os
 import re
 import sys
 
+import yaml
+
+try:
+    import xdg.BaseDirectory
+except ImportError:  # pragma: no cover
+    xdg_is_available = False
+else:  # pragma: no cover
+    xdg_is_available = True
+
 import cliapp
 from cliapp.genman import ManpageGenerator
 
@@ -46,6 +55,15 @@ class UnknownConfigVariable(cliapp.AppException):
         return self.msg
 
 
+class MalformedYamlConfig(cliapp.AppException):  # pragma: no cover
+
+    def __init__(self, msg):
+        cliapp.AppException.__init__(self, msg)
+
+    def __str__(self):  # pragma: no cover
+        return self.msg
+
+
 class Setting(object):
 
     action = 'store'
@@ -115,7 +133,10 @@ class StringListSetting(Setting):
         return self._strings
 
     def set_value(self, strings):
-        self._strings = strings
+        if type(strings) != list:
+            self._strings = [strings]
+        else:
+            self._strings = strings
         self.using_default_value = False
 
     def has_value(self):
@@ -285,6 +306,7 @@ class Settings(object):
     def __init__(self, progname, version, usage=None, description=None,
                  epilog=None):
         self._settingses = dict()
+        self._all_config_data = {}
         self._canonical_names = list()
 
         self.version = version
@@ -296,6 +318,7 @@ class Settings(object):
         self._add_default_settings()
 
         self._config_files = None
+        self._required_config_files = []
         self._cp = ConfigParser.ConfigParser()
 
     def _add_default_settings(self):
@@ -306,6 +329,7 @@ class Settings(object):
         self.string(['log'],
                     'write log entries to FILE (default is to not write log '
                     'files at all); use "syslog" to log to system log, '
+                    '"stderr" to log to the standard error output, '
                     'or "none" to disable logging',
                     metavar='FILE', group=log_group_name)
         self.choice(['log-level'],
@@ -487,6 +511,16 @@ class Settings(object):
 
         config_group = option_groups[config_group_name]
 
+        # Helper to add an option and add a reference to the Setting
+        # object it is created from (or None). This allows manpage
+        # generation to recognize when --foo and --no-foo come from
+        # the same setting.
+
+        def add_option_to_group(setting, group, *args, **kwargs):
+            option = group.add_option(*args, **kwargs)
+            option.from_setting = setting
+            return option
+
         # Return help text, unless setting/option is hidden, in which
         # case return optparse.SUPPRESS_HELP.
 
@@ -503,7 +537,8 @@ class Settings(object):
                 sys.stdout.write('%s\n' % name)
             sys.exit(0)
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--dump-setting-names',
             action='callback',
             nargs=0,
@@ -516,7 +551,8 @@ class Settings(object):
             self.dump_config(sys.stdout)
             sys.exit(0)
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--dump-config',
             action='callback',
             nargs=0,
@@ -527,8 +563,10 @@ class Settings(object):
 
         def reset_configs(option, opt_str, value, parser):
             self.config_files = []
+            self._required_config_files = []
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--no-default-configs',
             action='callback',
             nargs=0,
@@ -539,8 +577,10 @@ class Settings(object):
 
         def append_to_configs(option, opt_str, value, parser):
             self.config_files.append(value)
+            self._required_config_files.append(value)
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--config',
             action='callback',
             nargs=1,
@@ -556,7 +596,8 @@ class Settings(object):
                 print filename
             sys.exit(0)
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--list-config-files',
             action='callback',
             nargs=0,
@@ -567,7 +608,8 @@ class Settings(object):
 
         self._arg_synopsis = arg_synopsis
         self._cmd_synopsis = cmd_synopsis
-        p.add_option(
+        add_option_to_group(
+            None, p,
             '--generate-manpage',
             action='callback',
             nargs=1,
@@ -587,7 +629,8 @@ class Settings(object):
             sys.stdout.write(pp.format_help())
             sys.exit(0)
 
-        config_group.add_option(
+        add_option_to_group(
+            None, config_group,
             '--help-all',
             action='callback',
             help='show all options',
@@ -613,15 +656,16 @@ class Settings(object):
 
         def add_option(obj, s):
             option_names = self._option_names(s.names)
-            obj.add_option(*option_names,
-                           action='callback',
-                           callback=maybe(set_value),
-                           callback_args=(s,),
-                           type=s.type,
-                           nargs=s.nargs,
-                           choices=s.choices,
-                           help=help_text(s.help, s.hidden),
-                           metavar=s.metavar)
+            add_option_to_group(
+                s, obj, *option_names,
+                action='callback',
+                callback=maybe(set_value),
+                callback_args=(s,),
+                type=s.type,
+                nargs=s.nargs,
+                choices=s.choices,
+                help=help_text(s.help, s.hidden),
+                metavar=s.metavar)
 
         def add_negation_option(obj, s):
             option_names = self._option_names(s.names)
@@ -629,12 +673,13 @@ class Settings(object):
             neg_names = ['--no-' + x[2:] for x in long_names]
             unused_names = [x for x in neg_names
                             if x[2:] not in self._settingses]
-            obj.add_option(*unused_names,
-                           action='callback',
-                           callback=maybe(set_false),
-                           callback_args=(s,),
-                           type=s.type,
-                           help=help_text('', s.hidden))
+            add_option_to_group(
+                s, obj, *unused_names,
+                action='callback',
+                callback=maybe(set_false),
+                callback_args=(s,),
+                type=s.type,
+                help=help_text('opposite of %s' % option_names[0], s.hidden))
 
         # Add options for every setting.
 
@@ -695,20 +740,37 @@ class Settings(object):
         configs = []
 
         configs.append('/etc/%s.conf' % self.progname)
+        configs.append('/etc/%s.yaml' % self.progname)
         configs += self.listconfs('/etc/%s' % self.progname)
+
         configs.append(os.path.expanduser('~/.%s.conf' % self.progname))
+        configs.append(os.path.expanduser('~/.%s.yaml' % self.progname))
         configs += self.listconfs(
             os.path.expanduser('~/.config/%s' % self.progname))
 
+        # See <http://standards.freedesktop.org/basedir-spec/>. We
+        # support these if the xdg library is available. We always
+        # support the hardcoded locations so that people's config
+        # files don't get ignored just because the xdg library gets
+        # installed.
+
+        if xdg_is_available:  # pragma: no cover
+            for dirname in reversed(xdg.BaseDirectory.xdg_config_dirs):
+                pathname = os.path.join(dirname, self.progname)
+                for location in self.listconfs(pathname):
+                    if location not in configs:
+                        configs.append(location)
+
         return configs
 
     def listconfs(self, dirname, listdir=os.listdir):
         '''Return list of pathnames to config files in dirname.
 
-        Config files are expectd to have names ending in '.conf'.
+        Config files are expected to have names ending in '.conf' or
+        '.yaml'.
 
-        If dirname does not exist or is not a directory,
-        return empty list.
+        If dirname does not exist or is not a directory, return empty
+        list.
 
         '''
 
@@ -719,7 +781,7 @@ class Settings(object):
         basenames.sort(key=lambda s: [ord(c) for c in s])
         return [os.path.join(dirname, x)
                 for x in basenames
-                if x.endswith('.conf')]
+                if x.endswith('.conf') or x.endswith('.yaml')]
 
     def _get_config_files(self):
         if self._config_files is None:
@@ -746,26 +808,66 @@ class Settings(object):
 
         '''
 
-        cp = ConfigParser.ConfigParser()
-        cp.add_section('config')
+        self._all_config_data = {}
 
         for pathname in self.config_files:
             try:
                 f = open_file(pathname)
-            except IOError:
-                pass
-            else:
-                cp.readfp(f)
+                if pathname.endswith('.yaml'):
+                    self._read_yaml(pathname, f)
+                else:
+                    self._read_ini(pathname, f)
                 f.close()
+            except IOError:  # pragma: no cover
+                if pathname in self._required_config_files:
+                    raise
 
-                for name in cp.options('config'):
-                    value = cp.get('config', name)
-                    s = self.set_from_raw_string(pathname, name, value)
-                    if hasattr(s, 'using_default_value'):
-                        s.using_default_value = True
+    def _read_ini(self, pathname, f):
+        cp = ConfigParser.ConfigParser()
+        cp.add_section('config')
+        cp.readfp(f)
+        for name in cp.options('config'):
+            value = cp.get('config', name)
+            s = self.set_from_raw_string(pathname, name, value)
+            if hasattr(s, 'using_default_value'):
+                s.using_default_value = True
 
-        # Remember the ConfigParser for use in as_cp later on.
-        self._cp = cp
+        for section in [s for s in cp.sections() if s != 'config']:
+            if section not in self._all_config_data:
+                self._all_config_data[section] = {}
+            section_data = self._all_config_data[section]
+            for option in cp.options(section):
+                section_data[option] = cp.get(section, option)
+
+    def _read_yaml(self, pathname, f):
+        obj = yaml.safe_load(f)
+        self._check_yaml(pathname, obj)
+        config = obj.get('config', {})
+        for name, value in config.items():
+            if name not in self._settingses:
+                raise UnknownConfigVariable(pathname, name)
+            s = self._settingses[name]
+            s.set_value(value)
+            if hasattr(s, 'using_default_value'):
+                s.using_default_value = True
+
+        for section in [s for s in obj if s != 'config']:
+            if section not in self._all_config_data:
+                self._all_config_data[section] = {}
+            section_data = self._all_config_data[section]
+            for option in obj[section]:
+                section_data[option] = obj[section][option]
+
+    def _check_yaml(self, pathname, obj):  # pragma: no cover
+        if not isinstance(obj, dict):
+            raise cliapp.MalformedYamlConfig(
+                'Configuration file %s does not specify a key/value mapping' %
+                pathname)
+
+        if 'config' not in obj:
+            raise cliapp.MalformedYamlConfig(
+                'Configuration file %s does not have a "config" key' %
+                pathname)
 
     def _generate_manpage(self, o, dummy, value, p):  # pragma: no cover
         template = open(value).read()
@@ -782,16 +884,16 @@ class Settings(object):
         meanings it desires to the section names.
 
         '''
+
         cp = ConfigParser.ConfigParser()
         cp.add_section('config')
         for name in self._canonical_names:
             cp.set('config', name, self._settingses[name].format())
 
-        for section in self._cp.sections():
+        for section in self._all_config_data:
             if section != 'config':
                 cp.add_section(section)
-                for option in self._cp.options(section):
-                    value = self._cp.get(section, option)
+                for option, value in self._all_config_data[section].items():
                     cp.set(section, option, value)
 
         return cp
